@@ -1,11 +1,11 @@
 //! Format the output of syntect into an image
 use crate::error::FontError;
-use crate::font::{FontCollection, FontStyle};
+use crate::font::{FontCollection, FontStyle, TextLineDrawer};
 use crate::utils::*;
-use image::{DynamicImage, GenericImageView, Rgba, RgbaImage};
+use image::{Rgba, RgbaImage};
 use syntect::highlighting::{Color, Style, Theme};
 
-pub struct ImageFormatter {
+pub struct ImageFormatter<T> {
     /// pad between lines
     /// Default: 2
     line_pad: u32,
@@ -15,6 +15,9 @@ pub struct ImageFormatter {
     /// pad of top of the code area
     /// Default: 50
     code_pad_top: u32,
+    /// pad of right of the code area
+    /// Default: 25
+    code_pad_right: u32,
     /// Title bar padding
     /// Default: 15
     title_bar_pad: u32,
@@ -42,7 +45,7 @@ pub struct ImageFormatter {
     line_number_chars: u32,
     /// font of english character, should be mono space font
     /// Default: Hack (builtin)
-    font: FontCollection,
+    font: T,
     /// Highlight lines
     highlight_lines: Vec<u32>,
     /// Shadow adder
@@ -57,6 +60,8 @@ pub struct ImageFormatter {
 pub struct ImageFormatterBuilder<S> {
     /// Pad between lines
     line_pad: u32,
+    /// Padding to the right of the code
+    code_pad_right: u32,
     /// Show line number
     line_number: bool,
     /// Font of english character, should be mono space font
@@ -109,6 +114,12 @@ impl<S: AsRef<str> + Default> ImageFormatterBuilder<S> {
         self
     }
 
+    /// Set the pad on the right of the screen
+    pub fn code_pad_right(mut self, pad: u32) -> Self {
+        self.code_pad_right = pad;
+        self
+    }
+
     /// Set the font
     pub fn font(mut self, fonts: Vec<(S, f32)>) -> Self {
         self.font = fonts;
@@ -151,7 +162,7 @@ impl<S: AsRef<str> + Default> ImageFormatterBuilder<S> {
         self
     }
 
-    pub fn build(self) -> Result<ImageFormatter, FontError> {
+    pub fn build(self) -> Result<ImageFormatter<FontCollection>, FontError> {
         let font = if self.font.is_empty() {
             FontCollection::default()
         } else {
@@ -164,6 +175,7 @@ impl<S: AsRef<str> + Default> ImageFormatterBuilder<S> {
             line_pad: self.line_pad,
             code_pad: 25,
             code_pad_top: if title_bar { 50 } else { 0 },
+            code_pad_right: self.code_pad_right,
             title_bar_pad: 15,
             window_controls: self.window_controls,
             window_controls_width: 120,
@@ -191,38 +203,38 @@ struct Drawable {
     drawables: Vec<(u32, u32, Option<Color>, FontStyle, String)>,
 }
 
-impl ImageFormatter {
+impl<T: TextLineDrawer> ImageFormatter<T> {
     /// calculate the height of a line
-    fn get_line_height(&self) -> u32 {
-        self.font.get_font_height() + self.line_pad
+    fn get_line_height(&mut self) -> u32 {
+        self.font.height(" ") + self.line_pad
     }
 
     /// calculate the Y coordinate of a line
-    fn get_line_y(&self, lineno: u32) -> u32 {
+    fn get_line_y(&mut self, lineno: u32) -> u32 {
         lineno * self.get_line_height() + self.code_pad + self.code_pad_top
     }
 
     /// calculate the size of code area
-    fn get_image_size(&self, max_width: u32, lineno: u32) -> (u32, u32) {
+    fn get_image_size(&mut self, max_width: u32, lineno: u32) -> (u32, u32) {
         (
-            (max_width + self.code_pad).max(150),
+            (max_width + self.code_pad_right).max(150),
             self.get_line_y(lineno + 1) + self.code_pad,
         )
     }
 
     /// Calculate where code start
-    fn get_left_pad(&self) -> u32 {
+    fn get_left_pad(&mut self) -> u32 {
         self.code_pad
             + if self.line_number {
                 let tmp = format!("{:>width$}", 0, width = self.line_number_chars as usize);
-                2 * self.line_number_pad + self.font.get_text_len(&tmp)
+                2 * self.line_number_pad + self.font.width(&tmp)
             } else {
                 0
             }
     }
 
     /// create
-    fn create_drawables(&self, v: &[Vec<(Style, &str)>]) -> Drawable {
+    fn create_drawables(&mut self, v: &[Vec<(Style, &str)>]) -> Drawable {
         // tab should be replaced to whitespace so that it can be rendered correctly
         let tab = " ".repeat(self.tab_width as usize);
         let mut drawables = vec![];
@@ -246,7 +258,7 @@ impl ImageFormatter {
                     text.to_owned(),
                 ));
 
-                width += self.font.get_text_len(&text);
+                width += self.font.width(&text);
 
                 max_width = max_width.max(width);
             }
@@ -255,7 +267,7 @@ impl ImageFormatter {
 
         if self.window_title.is_some() {
             let title = self.window_title.as_ref().unwrap();
-            let title_width = self.font.get_text_len(title);
+            let title_width = self.font.width(title);
 
             let ctrls_offset = if self.window_controls {
                 self.window_controls_width + self.title_bar_pad
@@ -266,7 +278,7 @@ impl ImageFormatter {
 
             drawables.push((
                 ctrls_offset + self.title_bar_pad,
-                self.title_bar_pad + ctrls_center - self.font.get_font_height() / 2,
+                self.title_bar_pad + ctrls_center - self.font.height(" ") / 2,
                 None,
                 FontStyle::BOLD,
                 title.to_string(),
@@ -283,46 +295,47 @@ impl ImageFormatter {
         }
     }
 
-    fn draw_line_number(&self, image: &mut DynamicImage, lineno: u32, mut color: Rgba<u8>) {
+    fn draw_line_number(&mut self, image: &mut RgbaImage, lineno: u32, mut color: Rgba<u8>) {
         for i in color.0.iter_mut() {
             *i = (*i).saturating_sub(20);
         }
         for i in 0..=lineno {
-            let line_mumber = format!(
+            let line_number = format!(
                 "{:>width$}",
                 i + self.line_offset,
                 width = self.line_number_chars as usize
             );
-            self.font.draw_text_mut(
+            let y = self.get_line_y(i);
+            self.font.draw_text(
                 image,
                 color,
                 self.code_pad,
-                self.get_line_y(i),
+                y,
                 FontStyle::REGULAR,
-                &line_mumber,
+                &line_number,
             );
         }
     }
 
-    fn highlight_lines<I: IntoIterator<Item = u32>>(&self, image: &mut DynamicImage, lines: I) {
+    fn highlight_lines<I: IntoIterator<Item = u32>>(&mut self, image: &mut RgbaImage, lines: I) {
         let width = image.width();
-        let height = self.font.get_font_height() + self.line_pad;
-        let mut color = image.get_pixel(20, 20);
+        let height = self.get_line_height();
+        let color = image.get_pixel_mut(20, 20);
 
         for i in color.0.iter_mut() {
             *i = (*i).saturating_add(40);
         }
 
-        let shadow = RgbaImage::from_pixel(width, height, color);
+        let shadow = RgbaImage::from_pixel(width, height, *color);
 
         for i in lines {
             let y = self.get_line_y(i - 1);
-            copy_alpha(&shadow, image.as_mut_rgba8().unwrap(), 0, y);
+            copy_alpha(&shadow, image, 0, y);
         }
     }
 
     // TODO: use &T instead of &mut T ?
-    pub fn format(&mut self, v: &[Vec<(Style, &str)>], theme: &Theme) -> DynamicImage {
+    pub fn format(&mut self, v: &[Vec<(Style, &str)>], theme: &Theme) -> RgbaImage {
         if self.line_number {
             self.line_number_chars =
                 (((v.len() + self.line_offset as usize) as f32).log10() + 1.0).floor() as u32;
@@ -338,15 +351,15 @@ impl ImageFormatter {
         let foreground = theme.settings.foreground.unwrap();
         let background = theme.settings.background.unwrap();
 
-        let mut image =
-            DynamicImage::ImageRgba8(RgbaImage::from_pixel(size.0, size.1, background.to_rgba()));
+        let mut image = RgbaImage::from_pixel(size.0, size.1, background.to_rgba());
 
         if !self.highlight_lines.is_empty() {
             let highlight_lines = self
                 .highlight_lines
                 .iter()
                 .cloned()
-                .filter(|&n| n >= 1 && n <= drawables.max_lineno + 1);
+                .filter(|&n| n >= 1 && n <= drawables.max_lineno + 1)
+                .collect::<Vec<_>>();
             self.highlight_lines(&mut image, highlight_lines);
         }
         if self.line_number {
@@ -355,8 +368,7 @@ impl ImageFormatter {
 
         for (x, y, color, style, text) in drawables.drawables {
             let color = color.unwrap_or(foreground).to_rgba();
-            self.font
-                .draw_text_mut(&mut image, color, x, y, style, &text);
+            self.font.draw_text(&mut image, color, x, y, style, &text);
         }
 
         if self.window_controls {
